@@ -9,7 +9,6 @@ use Future::Utils;
 use IO::Async::Loop;
 use IO::Async::Signal;
 use IO::Async::Stream;
-use IO::Async::Timer::Periodic;
 use IO::Handle;
 use Scalar::Util qw(blessed);
 
@@ -54,61 +53,6 @@ sub run
 {
     my ($self) = @_;
 
-    $self->{client_requests}  = Future::Queue->new();
-    $self->{client_responses} = Future::Queue->new();
-    $self->{server_requests}  = Future::Queue->new();
-    $self->{server_responses} = Future::Queue->new();
-
-    Future::Utils::repeat
-    {
-        $self->{client_requests}->shift->on_done(
-            sub {
-                my ($request) = @_;
-
-                $self->handle_client_request($request);
-                return;
-            }
-        );
-    } ## end Future::Utils::repeat
-    while => sub { 1 };
-
-    Future::Utils::repeat
-    {
-        $self->{client_responses}->shift->on_done(
-            sub {
-                my ($response) = @_;
-
-                $self->handle_client_response($response);
-                return;
-            }
-        );
-    } ## end Future::Utils::repeat
-    while => sub { 1 };
-
-    Future::Utils::repeat
-    {
-        $self->{server_requests}->shift->on_done(
-            sub {
-                my ($request) = @_;
-                $self->handle_server_request($request);
-                return;
-            }
-        );
-    } ## end Future::Utils::repeat
-    while => sub { 1 };
-
-    Future::Utils::repeat
-    {
-        $self->{server_responses}->shift->on_done(
-            sub {
-                my ($response) = @_;
-                $self->handle_server_response($response);
-                return;
-            }
-        );
-    } ## end Future::Utils::repeat
-    while => sub { 1 };
-
     $self->{stream} = IO::Async::Stream->new_for_stdio(
         autoflush => 0,
         on_read   => sub {
@@ -164,7 +108,7 @@ sub handle_client_message
 
         if (blessed($message) and $message->isa('PLS::Server::Response'))
         {
-            $self->{server_responses}->push($message);
+            $self->send_message($message);
             return;
         }
     } ## end if (length $message->{...})
@@ -177,11 +121,11 @@ sub handle_client_message
 
     if ($message->isa('PLS::Server::Request'))
     {
-        $self->{client_requests}->push($message);
+        $self->handle_client_request($message);
     }
     if ($message->isa('PLS::Server::Response'))
     {
-        $self->{client_responses}->push($message);
+        $self->handle_client_response($message);
     }
 
     return;
@@ -195,7 +139,7 @@ sub send_server_request
 
     if ($request->isa('PLS::Server::Request'))
     {
-        $self->{server_requests}->push($request);
+        $self->handle_server_request($request);
     }
     elsif ($request->isa('Future'))
     {
@@ -203,7 +147,7 @@ sub send_server_request
             sub {
                 my ($request) = @_;
 
-                $self->{server_requests}->push($request);
+                $self->handle_server_request($request);
             }
         )->retain();
     } ## end elsif ($request->isa('Future'...))
@@ -232,7 +176,7 @@ sub handle_client_request
     {
         if ($response->isa('PLS::Server::Response'))
         {
-            $self->{server_responses}->push($response);
+            $self->send_message($response);
         }
         elsif ($response->isa('Future'))
         {
@@ -241,11 +185,11 @@ sub handle_client_request
             $response->on_done(
                 sub {
                     my ($response) = @_;
-                    $self->{server_responses}->push($response);
+                    $self->send_message($response);
                 }
               )->on_cancel(
                 sub {
-                    $self->{server_responses}->push(PLS::Server::Response::Cancelled->new(id => $request->{id}));
+                    $self->send_message(PLS::Server::Response::Cancelled->new(id => $request->{id}));
                 }
               );
         } ## end elsif ($response->isa('Future'...))
@@ -295,24 +239,6 @@ sub handle_server_response
     return;
 } ## end sub handle_server_response
 
-sub monitor_client_process
-{
-    my ($self, $pid) = @_;
-
-    my $timer = IO::Async::Timer::Periodic->new(
-        interval => 30,
-        on_tick  => sub {
-            return if (kill 'ZERO', $pid);
-            $self->stop(1);
-        }
-    );
-    $self->{loop}->add($timer);
-
-    $timer->start();
-
-    return;
-} ## end sub monitor_client_process
-
 sub stop
 {
     my ($self, $exit_code) = @_;
